% 
clear all;
UsedMeth = [1 2 3]; 
MethName={'SGD-ADMM','RDA-ADMM','SDCA-ADMM'}; 

ss=[10];  %sub-batch size
eta = 0.01;
rho = 1;
rho_sdca = 0.1;
b_average = 1;
lossfun = 'shinge';
maxiter=1000000;
max_storelen = 200;

NUMITER = 1;
max_time = 100; %Execution time in seconds.

DATA_TYPE = 4;
if DATA_TYPE == 3
    b_rand_perm = 0;
else
    b_rand_perm = 1;
end;
b_showfigure = 1;

for srandnum = 1:NUMITER
    rand('state',srandnum);
    randn('state',srandnum);

    m = 512; 
    lambdascale=0.01;

    n = 32*32; kk = 1; k=kk*sqrt(n); A=randn(m,n);             
    w0=randblocksparse(sqrt(n),kk); yy=sign(A*w0+3*randn(m,1));

    lambda=lambdascale*max(abs(A'*yy));

    Acon.times = @(x)([x;x]);
    Acon.Ttimes = @(x)(x(1:length(x)/2)+x((length(x)/2+1):length(x)));
    Acon.TTtimes = @(x)(2*x);
    Acon.maxeig = 2; 
    Acon.size = [2*n n];

    Adesign.size = [m n];
    Adesign.gen = @(ssin,b_rand_perm,kin,I_all)(basic_A(A,ssin,b_rand_perm,kin,I_all));
    Adesign.maxnorm = max(sqrt(sum(A.^2,2)));

    hsm = 0.01;
    STfunction = @(vvv,lambdav)(blockST(vvv/(1+hsm*lambdav),lambdav/(1+hsm*lambdav)));
    regfunction = @(x)(block_reg(x)+hsm*norm(x)^2/2*2);
    STfunction_scale = @(vvv,lambdav,sc)(blockST(vvv/(1+hsm*lambdav*sc),lambdav/(1+hsm*lambdav*sc)));
            
    b_rand_perm_first = 0;            


    out_kizami = max(ceil(maxiter/1000),2);

    fval=cell(length(UsedMeth),1);
    time=cell(length(UsedMeth),1);
    test_err=cell(length(UsedMeth),1);
    for ii=1:length(ss)                          
        for meth=UsedMeth
            switch meth
                 case {1,2} %stochastic-ADMMs
                    etain = eta;
                    [xx,xbar,stat]=sADMM(zeros(n,1),Adesign,yy,Acon,STfunction,regfunction,lambda,'ss',ss(ii),'maxiter', ...
                                maxiter,'eta',etain,'rho',rho,'b_average',b_average,'out_kizami',out_kizami,'rand_perm', ...
                                b_rand_perm,'b_rand_perm_first',b_rand_perm_first,'max_time',max_time,...,
                                'disp_message',sprintf('DATA_TYPE: %d',DATA_TYPE),'lossfun',lossfun,'type',meth);       
                case {3}
                    [xx,xbar,stat,sdcaopt]=sdcaADMM(Adesign,yy,Acon,STfunction_scale,lambda,'ss',ss(ii),'maxiter',maxiter,'max_time',max_time, ...
                                'rho',rho_sdca,'b_average',0,'rand_perm',b_rand_perm,'disp_message',sprintf('DATA_TYPE: %d',DATA_TYPE),'lossfun',lossfun,'max_storelen',max_storelen); 
            end;

            %time
            time{meth}(ii,1:length(stat.time))=stat.time;

            if strcmp(lossfun,'logit')
                ploss = @(fin,yin)(loss_lrp(fin,yin));
            elseif strcmp(lossfun,'shinge')
                ploss = @(fin,yin)(loss_shp_scale(fin,yin,1));
            end;

            %fval
            out=A*xx;
            for jj=1:size(out,2)
                %[floss,gloss]=loss_lrp(out(:,jj), yy);
                [floss,gloss]=ploss(out(:,jj), yy);
                fval{meth}(ii,jj)=floss+lambda*regfunction(xx(:,jj));%sum(abs(xx(:,jj)));
            end

            %test_err          
            Atest=randn(10000,n);
            ytest = sign(Atest*w0);
            out=sign(Atest*xx);
            for jj=1:size(out,2)
                test_err{meth}(ii,jj) = mean(ytest~=out(:,jj));
            end;

        end;
    end
    STORERESULTS{srandnum}.fval = fval;
    STORERESULTS{srandnum}.test_err = test_err;
    STORERESULTS{srandnum}.time=time;
end;

if b_showfigure
    figure(1), 
    clf;
    hold off;
    markertype ={'.','o','x','+','*','s','d','.','o','x','+','*','s','d','.','o','x','+','*','s','d','.','o','x','+','*','s','d'};
    labcount = 0;
    for meth=UsedMeth
        vec = fval{meth}(ii,:);
        tvec = time{meth}(ii,:);
        mkt_ind = mod(meth-1,length(markertype))+1;
        plot(tvec', vec(1:length(tvec))', ['-' markertype{mkt_ind}], 'linewidth', 2);    
        hold all;

        labcount = labcount + 1;
        label{labcount} = sprintf('s=%d,%s',ss(ii),MethName{meth});
    end;
    grid on;
    set(gca,'fontsize',14);
    set(gca,'Xscale','log');
    set(gca,'Yscale','log');
    xlabel('CPU time')
    ylabel('f(w^t)')
    legend(label);
    title(sprintf('eta_1=%g',eta));
end;
clear A Atest;