function [xx,xbar,stat]=sADMM(xx,AA,ytr,A,STfunction,regfunction,lambda,varargin)
%type = 1: SGD-ADMM
%type = 2: RDA-ADMM
%
%xx:primal variable 1, yy:primal variable 2
%A*xx - yy = 0;
%zz:dual variable

opt=propertylist2struct(varargin{:});
opt=set_defaults(opt, 'ss', 100,'eta',0.001,'rho',1,'tol',1e-3,'w0',[],'maxiter',100000,'rand_perm',1,'type',1,...
    'b_average',1,'out_kizemi',10,'radius',20,'b_rand_perm_first',0,'max_time',10^10,'disp_message',' ','lossfun','logit','max_storelen',500);


rho = opt.rho;
type = opt.type;
lossfun = opt.lossfun;

lsing = A.maxeig;%eigs(A'*A,1);
if 1/opt.eta < rho*lsing && (type == 1 || type == 2 || type >= 100)
    opt.eta = 1/(2*lsing*rho);%1/2*lsing*rho;
end;

b_average = opt.b_average;

rho = opt.rho;

n = AA.size(1); m = AA.size(2); %[n,m]=size(AA);
tmp=A.size; %size(A);
dd = tmp(1);

if opt.b_rand_perm_first
    I_all = randperm(n);
else
    I_all = 1:n;
end;

out_kizemi = opt.out_kizemi;
if type >= 100
    xout = [xx, zeros(m,min(1000,ceil(opt.maxiter/out_kizemi)-2))]; %[xx, zeros(n,opt.maxiter-1)];
else
    xout = [xx, zeros(m,ceil(opt.maxiter/out_kizemi)-2)]; %[xx, zeros(n,opt.maxiter-1)];\
end;
gbar = zeros(m,1);
time = zeros(1,size(xout,2));

time0=cputime;
tic;

if type >= 100
    opt.ss = n;
end;

%AAT = A*A';

zz = zeros(dd,1);
zbar = zeros(dd,1);
yy = zeros(dd,1);
ybar = zeros(dd,1);
xbar = xx;
%vv = zeros(m,1);


if strcmp(lossfun,'logit')
    ploss_scale = @(xin,yin,scale)loss_lrp_scale(xin,yin,scale);
    dloss_scale = @(xin,yin,scale)loss_lrd_scale(xin,-yin,scale); 
elseif strcmp(lossfun,'shinge')
    ploss_scale = @(xin,yin,scale)loss_shp_scale(xin,yin,scale);
    dloss_scale = @(xin,yin,scale)loss_shd_scale(xin,yin,scale);    
end;

ind_stored = 1;
last_stored_time = 0;
kizami_time = opt.max_time/opt.max_storelen;


%%%%%%!!!!!!!!!!!!!!!!!!!!!!
if type >= 100
    [As,I,I_all] = AA.gen(opt.ss,opt.rand_perm,1,I_all); 
else
    A_orig_t = AA.gen(n,1,1,I_all);
    A_orig_t = A_orig_t';
end;
%%%%%%!!!!!!!!!!!!!!!!!!!!!!



kk=1;
floss = 0;
fregret = 0;
while kk<opt.maxiter
    if type == 1 || type == 3 || type == 5 
        eta = opt.eta/sqrt(kk);
    elseif type == 104
        eta = opt.eta/kk^(1/10);
    elseif type == 105
        eta = opt.eta*kk/kk^(1/10);
    elseif type >= 100 && type <= 104
        eta = opt.eta;        
    else
        eta = opt.eta*sqrt(kk);
    end;
    
    %[As,I,I_all] = AA.gen(opt.ss,opt.rand_perm,kk,I_all);
      if type < 100
        %%%%%%!!!!!!!!!!!!!!!!!!!!!!
        ix0 = ceil(n*rand);
        if ix0<=n-opt.ss+1
            ix=ix0:ix0+opt.ss-1;
        else
            ix=[1:(ix0+opt.ss-n-1), ix0:n];
        end
        I=sort(ix);
        As = A_orig_t(:,I)';
        %%%%%%!!!!!!!!!!!!!!!!!!!!!!
    end;

    [ff,gg]=ploss_scale(As*xx, ytr(I), 1);

    gg= As'*gg*(n/opt.ss);
    if type == 2 || type == 4 || type == 105 %RDA type
        gbar = ((kk-1)*gbar + gg)/kk;
    end;

    switch type 
        case 1 %SGD-ADMM            
            xx = - (gg - A.Ttimes(zz + rho*yy))*eta + (xx - rho*eta*(A.TTtimes(xx)));
            normxx = norm(xx);
            if normxx >0, xx = xx*min(normxx,opt.radius)/normxx; end;
        case 2 %RDA-ADMM
            xx = - (gbar - A.Ttimes(zbar + rho*ybar) + rho*A.TTtimes(xbar))*eta;
            normxx = norm(xx);
            if normxx >0, xx = xx*min(normxx,opt.radius)/normxx; end;
    end;
    yy = STfunction((A.times(xx) - zz/rho),lambda/rho);
    zz = zz - rho*(A.times(xx)-yy);

    xbar = ((kk-1)*xbar + xx)/kk;
    ybar = ((kk-1)*ybar + yy)/kk;
    zbar = ((kk-1)*zbar + zz)/kk;

    fobj = ff/opt.ss*n + lambda*regfunction(xx);
    if type >= 100
        fval(kk) = fobj;
    else
        fregret = ((kk-1)*fregret + fobj)/kk;    
        fval(kk) = fregret; 
    end;

    fprintf('[%d] fval=%g floss=%g eta=%g %s\n', kk, fval(kk), ff, eta, opt.disp_message);

    if mod(kk,out_kizemi) == 1
        ind = (kk-1)/out_kizemi+2;
        if b_average
            xout(:,ind)=xbar;%xbar;%xx;
            time(ind)=toc;%cputime-time0;
        else
            xout(:,ind)=xx;
            time(ind)=toc;%cputime-time0;
        end;
    end;

   
    if toc >= opt.max_time
        ind = floor((kk-1)/out_kizemi)+2;
        if b_average
            xout(:,ind+1)=xbar;
        else
            xout(:,ind+1)=xx;
        end;
        xout=xout(:,1:(ind+1));
        time(ind+1) = toc;
        time=time(1:(ind+1));
        break;
    end;

    
    kk=kk+1;
end

stat=struct('time', time); 

xx=xout;

function vv=softth(vv,lambda)
n = size(vv,1);

Ip=find(vv>lambda);
In=find(vv<-lambda);

vv=sparse([Ip;In],1,[vv(Ip)-lambda;vv(In)+lambda],n,1);


function [fdloss grad Hessian hmin] = loss_lrd_scale(xin,yin,scale)
[fdloss grad Hessian hmin] = loss_lrd(xin/scale,yin);
fdloss = fdloss*scale;
%grad = grad;
Hessian = Hessian/scale;
hmin = hmin/scale;

function [floss, gloss]=loss_lrp_scale(zz, yy, scale)

zy      = zz.*yy;
z2      = 0.5*[zy, -zy];
outmax  = max(z2,[],2);
sumexp  = sum(exp(z2-outmax(:,[1,1])),2);
logpout = z2-(outmax+log(sumexp))*ones(1,2);
pout    = exp(logpout);

floss   = -sum(logpout(:,1))*scale;
gloss   = -yy.*pout(:,2)*scale;
%Hessian = 

function dval = evaldual(aa, ATaa, yy, lambda)

mm=length(yy);

dnm = max(abs(ATaa));

if dnm>0
  aa  = min(1, lambda/dnm)*aa;
end

ya=yy.*aa;

I = find(0<ya & ya<1);
dval = sum((1-ya(I)).*log(1-ya(I))+ya(I).*log(ya(I)));

function [floss, gloss]=loss_lrp(zz, yy)

zy      = zz.*yy;
z2      = 0.5*[zy, -zy];
outmax  = max(z2,[],2);
sumexp  = sum(exp(z2-outmax(:,[1,1])),2);
logpout = z2-(outmax+log(sumexp))*ones(1,2);
pout    = exp(logpout);

floss   = -sum(logpout(:,1));
gloss   = -yy.*pout(:,2);