function [lamout,lambdabar,stat,opt]=sdcaADMM(AA,ytr,B,STfunction_scale,Creg,varargin)
%min_{x,y} (\sum_i l^*(y_i,x))/n + (Creg*psi)^*(y)  s.t. A*x + B*y = 0
%STfunction_scale(q,C,e) = argmin_y(|q-y|^2/2 + (C/e)*psi(y*e))
%sdcaADMM written by Taiji Suzuki. 
%Tokyo Institute of Technology. E-mail: s-taiji@is.titech.ac.jp
opt=propertylist2struct(varargin{:});
opt=set_defaults(opt, 'ss', 100,'etaB',2,'etaA',2,'rho',1,'tol',1e-3,'w0',[],'maxiter',100000,'rand_perm',1,'type',1,...
    'b_average',0,'out_kizemi',10,'radius',20,'b_rand_perm_first',2,'max_time',10^10,'disp_message',' ','lossfun','logit','max_storelen',50);


n = AA.size(1); p = AA.size(2); 
tmp=B.size; %size(A);
dd = tmp(1); 
lossfun = opt.lossfun;


Beig = B.maxeig;
if opt.etaB <= 2*Beig 
    opt.etaB = Beig*1.1; %2;
end;
Anorm = AA.maxnorm;
if opt.etaA <= ((Anorm^2)*(opt.ss)) %2*Anorm
    opt.etaA = (Anorm^2)*(opt.ss) + 1;
end;

b_average = opt.b_average;



rho = opt.rho;
gamma = 1/(2*n); %1/(n); 
opt.gamma = gamma;
I_all = 1:n;


xx = zeros(n,1);
yy = zeros(dd,1);
lambda = zeros(p,1);
lambdabar = lambda;
Ax = zeros(p,1);
By = zeros(p,1);

lamout = [lambda, zeros(p,opt.max_storelen)]; 

time = zeros(1,size(lamout,2));

time0=cputime;
tic;

ind_stored = 1;
last_stored_time = 0;
chopped_time = opt.max_time/opt.max_storelen;

A_orig_t = AA.gen(n,0,1,I_all); %AA.gen(n,1,1,I_all);
A_orig_t = A_orig_t';
KK = ceil(n/opt.ss);

etaA = opt.etaA;
kk=1;
while kk<opt.maxiter
    
    etaB = opt.etaB;
    
    %[As,I,I_all] = AA.gen(opt.ss,4,kk,I_all);%opt.rand_perm,kk,I_all);
    
    blockrandtype = 2;
    if blockrandtype  == 1
        block_ind = ceil(rand*KK);
        ix0 = (block_ind-1)*opt.ss + 1;
        ix = ix0:min(n,ix0+opt.ss-1);
    else
        ix0 = ceil(n*rand);
        if ix0<=n-opt.ss+1
            ix=ix0:ix0+opt.ss-1;
        else
            ix=[1:(ix0+opt.ss-n-1), ix0:n];
        end
    end;
    I=sort(ix);
    As = A_orig_t(:,I)';
   
    % y-update
    qin = yy + B.times(lambda - rho*(Ax+By))/(rho*etaB);
    p_y = STfunction_scale(qin,Creg,rho*etaB); %STfunction_scale(q,C,e) = argmin_y(|q-y|^2/2 + (C/e)*psi(y*e))
    yy = qin - p_y;
    Bytmp = B.Ttimes(yy);
    Byhat = n*Bytmp - (n-n/KK)*By;
    By = Bytmp;

    type = 2;
    % x-update
    switch type 
        case {1} 
            eta_scale = 1.1;
            if kk == 1
                etaA = eta_scale*norm(As*As',Inf); 
            end;
            tmp_vec = As*(lambda -rho*(Ax+By));
            eta_power_max = 10;
            for eta_power = 1:eta_power_max
                qin = xx(I) + tmp_vec/(rho*etaA); 

                if strcmp(lossfun,'logit')
                    scale = 1;
                    ploss = @(xin,yin)loss_lrp(xin,yin)*scale;   
                    dloss = @(xin,yin)loss_lrd_scale(xin,-yin,scale); 

                    b_Hessian = 1;
                    if sum(xx(I) == 0)
                        [primalval dualval] = Newton_loss_sq(ploss,dloss,ytr(I),eye(length(I)),(rho*etaA),qin,'logit',scale,[],b_Hessian); 
                    else
                        [primalval dualval] = Newton_loss_sq(ploss,dloss,ytr(I),eye(length(I)),(rho*etaA),qin,'logit',scale,xx(I),b_Hessian);
                    end;
                    xi = dualval;
                elseif strcmp(lossfun,'shinge')
                    xi = loss_shd_prox_scale(ytr(I),qin,rho*etaA); 
                end;
                dxx = As'*(xi - xx(I));
                if norm(dxx)^2 < eta_scale*etaA*norm(xi - xx(I))^2
                    break;
                else                    
                    if eta_power == eta_power_max-1
                        etaA = eta_scale*norm(As*As',Inf); 
                    elseif eta_power < eta_power_max-1
                        etaA = etaA*(1.2);
                    end;
                end;
            end;                            
            etaA = etaA*0.8;            
            Axhat = (n/KK)*Ax + n*dxx; 
            Ax = Ax + dxx;
            xx(I) = xi;
        case {2}
            eta_scale = 1.1;
            etaA = eta_scale*norm(As*As',Inf); 
            tmp_vec = As*(lambda -rho*(Ax+By));
            qin = xx(I) + tmp_vec/(rho*etaA); 

            if strcmp(lossfun,'logit')
                scale = 1;
                ploss = @(xin,yin)loss_lrp(xin,yin)*scale;   
                dloss = @(xin,yin)loss_lrd_scale(xin,-yin,scale); 

                b_Hessian = 1;
                if sum(xx(I) == 0)
                    [primalval dualval] = Newton_loss_sq(ploss,dloss,ytr(I),eye(length(I)),(rho*etaA),qin,'logit',scale,[],b_Hessian); 
                else
                    [primalval dualval] = Newton_loss_sq(ploss,dloss,ytr(I),eye(length(I)),(rho*etaA),qin,'logit',scale,xx(I),b_Hessian);
                end;
                xi = dualval;
            elseif strcmp(lossfun,'shinge')
                xi = loss_shd_prox_scale(ytr(I),qin,rho*etaA); 
            end;
            dxx = As'*(xi - xx(I));
            Axhat = (n/KK)*Ax + n*dxx; 
            Ax = Ax + dxx;
            xx(I) = xi;
    end;

    % lambda-update    
    lambda = lambda - (gamma*rho)*(Axhat+Byhat); 
    if b_average
        lambdabar = ((kk-1)*lambdabar + lambda)/kk;
    end;
    
    fprintf('[%d] SDCA %s\n', kk, opt.disp_message);

    current_time = toc;
    if (current_time - last_stored_time > chopped_time) || kk == 1 
        ind = ind_stored + 1; 
        if b_average
            lamout(:,ind)=lambdabar;
            time(ind)=current_time;
        else
            lamout(:,ind)=lambda;
            time(ind)=current_time;
        end;
        last_stored_time = time(ind);
        ind_stored = ind;
    end;

    if current_time >= opt.max_time        
        break;
    end;
    
    kk=kk+1;
end

ind = ind_stored; 
if b_average
    lamout(:,ind+1)=lambdabar;
else
    lamout(:,ind+1)=lambda;
end;
lamout=lamout(:,1:(ind+1));
time(ind+1) = current_time;
time=time(1:(ind+1));

% Check duality gap
%[floss, gloss]=loss_lrp(AA*xbar, ytr);
%fval(end+1) = floss + lambda*regfunction(xx);
%dval = evaldual(-gloss, -AA'*gloss, ytr, lambda);
%res = (fval(end)-(-dval))/fval(end);

stat=struct('time', time); 
opt.type = type;
opt.blockrandtype = blockrandtype;


function vv=softth(vv,lambda)
n = size(vv,1);

Ip=find(vv>lambda);
In=find(vv<-lambda);

vv=sparse([Ip;In],1,[vv(Ip)-lambda;vv(In)+lambda],n,1);


function [fdloss grad Hessian hmin] = loss_lrd_scale(xin,yin,scale)
[fdloss grad Hessian hmin] = loss_lrd(xin/scale,yin);
fdloss = fdloss*scale;
%grad = grad;
Hessian = Hessian/scale;
hmin = hmin/scale;

function [floss, gloss]=loss_lrp_scale(zz, yy, scale)

zy      = zz.*yy;
z2      = 0.5*[zy, -zy];
outmax  = max(z2,[],2);
sumexp  = sum(exp(z2-outmax(:,[1,1])),2);
logpout = z2-(outmax+log(sumexp))*ones(1,2);
pout    = exp(logpout);

floss   = -sum(logpout(:,1))*scale;
gloss   = -yy.*pout(:,2)*scale;
%Hessian = 

function dval = evaldual(aa, ATaa, yy, lambda)

mm=length(yy);

dnm = max(abs(ATaa));

if dnm>0
  aa  = min(1, lambda/dnm)*aa;
end

ya=yy.*aa;

I = find(0<ya & ya<1);
dval = sum((1-ya(I)).*log(1-ya(I))+ya(I).*log(ya(I)));
